import abc

from controller import Radar
from flockai.interfaces.drone import IDrone


class AutopilotControlledDrone(IDrone, abc.ABC):
    def __init__(self, devices):
        super().__init__(devices)
        self.multiplier = 1
        self.target_id_count = 0

    def get_input(self):
        """
        The drone rotates until it identifies a target.
        When a target is identified, the drone moves forward
        :return: A tuple with pitch, roll, yaw disturbance to actuate the drone
        """
        radar: Radar = self.devices['radar']['device']
        total_targets = radar.getNumberOfTargets()
        targets = radar.getTargets()

        if total_targets > 0:
            self.target_id_count += 1
            for i in range(total_targets):
                print(f'--target {i}: distance = {targets[i].distance} azimuth = {targets[i].azimuth}')
                if targets[i].distance < 2.5:
                    print('emitting')
                    emitter = self.devices['emitter']['device']
                    message = 'DESTINATION_ARRIVED'
                    emitter.send(message.encode('utf-8'))

            if self.target_id_count == 1:
                self.multiplier *= -1

            roll_disturbance, pitch_disturbance, yaw_disturbance = 0, 2, 0
        else:
            self.target_id_count = 0
            roll_disturbance, pitch_disturbance, yaw_disturbance = 0, 0, self.multiplier * 0.4

        return roll_disturbance, pitch_disturbance, yaw_disturbance



import abc


class FlockAIClassifier(abc.ABC):
    def __init__(self):
        self.model = None

    @abc.abstractmethod
    def _load_model(self):
        raise NotImplementedError

    @abc.abstractmethod
    def predict(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _get_model_input(self):
        raise NotImplementedError




class Battery:
    DJI_CAPACITY = 3850  # mAh
    DJI_VOLTAGE = 15.4  # V
    DJI_ENERGY = 59.29  # Wh
    DJI_MAX_FLIGHT_TIME = 31 * 60  # seconds
    DJI_MAX_HOVERING_TIME = 29 * 60  # seconds
    DJI_SAFE_LANDING_PERCENTAGE = 0.1  # percentage

    def __init__(self):
        self.capacity = 0
        self.voltage = 0
        self.energy = 0
        self.max_flight_time = 0
        self.max_hover_time = 0
        self.safe_landing_percentage = 0

